% Leave-one-out validation for quasi-conformal statistical shape analysis
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, H. L. Chan, R. Yong, S. Ranjitkar, A. Brook, G. Townsend, K. Chen, and L. M. Lui, 
%     "Tooth morphometry using quasi-conformal theory."
%     Pattern Recognition, 99, 107064, 2020.
%
% Copyright (c) 2019, Hei Long Chan

function [ TPR, TNR, CA ] = QCSSA_Classify( Energy_cluster1, Energy_cluster2, fold, method )

% Initialize parameters
cluster1_total = size(Energy_cluster1,1);
cluster2_total = size(Energy_cluster2,1);
n = cluster1_total + cluster2_total;
TPR = 0; TNR = 0;
if fold==1
    fold = 'n';
end

% Compute the folding of data for validation
if ~strcmp(fold,'n')
    permutation1 = randperm(size(Energy_cluster1,1));
    permutation2 = randperm(size(Energy_cluster2,1));
    grouping1 = zeros(ceil(length(permutation1)/fold),fold);
    for ii = 1:size(grouping1,1)-1
        grouping1(ii,:) = permutation1(1:fold);
        permutation1(1:fold) = [];
    end
    grouping1(end,1:length(permutation1)) = permutation1;
    grouping2 = zeros(ceil(length(permutation2)/fold),fold);
    for ii = 1:size(grouping2,1)-1
        grouping2(ii,:) = permutation2(1:fold);
        permutation2(1:fold) = [];
    end
    grouping2(end,1:length(permutation2)) = permutation2;
    iter = fold;
else
    iter = n;
end

% Classification
for i = 1:iter
    if strcmp(fold,'n')
        train_ind1 = 1:size(Energy_cluster1,1);
        train_ind2 = 1:size(Energy_cluster2,1);
        if i<=cluster1_total
            train_ind1(i) = [];
            test_ind1 = i;
            test_ind2 = [];
        else
            train_ind2(i-cluster1_total) = [];
            test_ind1 = [];
            test_ind2 = i-cluster1_total;
        end
    else
        test_ind1 = grouping1(:,i);
        test_ind2 = grouping2(:,i);
        train_ind1 = grouping1; train_ind1(:,i) = [];
        train_ind2 = grouping2; train_ind2(:,i) = [];
    end
    E_c1 = Energy_cluster1(train_ind1(:),:);
    E_c2 = Energy_cluster2(train_ind2(:),:);
    E_test = [Energy_cluster1(test_ind1,:);Energy_cluster2(test_ind2,:)];
    label = [zeros(length(test_ind1),1);ones(length(test_ind2),1)];
    switch(method)
        case 'l2'
            % Learn the best chopping criterion
            [Model,criterion] = Threshold_criterion(E_c1,E_c2);
            % Import the test subject
            for j = 1:length(label)
                SubjectDistance = sqrt(sum((E_test(j,:)-Model).^2,2));
                if (label(j)==0)&&(SubjectDistance<=max(criterion))
                    TPR = TPR + 1;
                elseif (label(j)==1)&&(SubjectDistance>min(criterion))
                    TNR = TNR + 1;
                end
            end
        case 'svm'
            % SVM clustering
            a = fitcsvm([E_c1;E_c2],[ones(size(E_c1,1),1);-ones(size(E_c2,1),1)]);
            % Import the test subject
            for j = 1:length(label)
                test = predict(a,E_test(j,:));
                if (label(j)==0)&&(test==1)
                    TPR = TPR + 1;
                elseif (label(j)==1)&&(test~=1)
                    TNR = TNR + 1;
                end
            end
    end
end

% Compute the statistical result
CA = (TPR+TNR) / n;
TPR = TPR / cluster1_total;
TNR = TNR / cluster2_total;






%% Threshold criterion
function [Model,criterion] = Threshold_criterion( Cluster1, Cluster2 )

% Initialize parameters
n1 = size(Cluster1,1);

% Compute the feature vector
Model = mean(Cluster1,1);

% Compute the 2-norm distance
distance = sqrt(sum(bsxfun(@minus,[Cluster1;Cluster2],Model).^2,2));

% Learn the best chopping criterion
[distance_sorted,sortindex] = sort(distance);
sortindex(sortindex>n1) = -1;
sortindex(sortindex>0)  =  1;
bestchop = separate(sortindex');
try
    criterion = (distance_sorted(bestchop)+distance_sorted(bestchop+1))/2;
catch
    criterion = distance_sorted(bestchop);
end
%%