% Compute p-value for quasi-conformal statistical shape analysis
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, H. L. Chan, R. Yong, S. Ranjitkar, A. Brook, G. Townsend, K. Chen, and L. M. Lui, 
%     "Tooth morphometry using quasi-conformal theory."
%     Pattern Recognition, 99, 107064, 2020.
%
% Copyright (c) 2019, Hei Long Chan

function P = QCSSA_Pvalue( Energy_cluster1 , Energy_cluster2 )

% Intialize parameters
m = size(Energy_cluster1,1);   n = size(Energy_cluster2,1);
m1 = m-1;   n1 = n-1;
sumH = sum(Energy_cluster1,1);   sumU = sum(Energy_cluster2,1);
meanH = sumH/m;   meanU = sumU/n;
dfe = m+n-3;

% Compute variance of the energy
temp1 = bsxfun(@minus, Energy_cluster1, meanH);
varH  = sum(temp1.^2,1)./m1;
temp2 = bsxfun(@minus, Energy_cluster2, meanU);
varU  = sum(temp2.^2,1)./n1;

% Compute all the p-value (i-th row is the p-value taking the i-th subject out)
P = zeros(m+n,size(Energy_cluster1,2));
E = Energy_cluster1;
for i = 1:m
    % Compute p-value
    E_mean = (sumH-Energy_cluster1(i,:))./m1;
    if i==1
        temp = bsxfun(@minus, E(2:end,:), E_mean);
    elseif i==m
        temp = bsxfun(@minus, E(1:end-1,:), E_mean);
    else
        temp = bsxfun(@minus, [E(1:i-1,:);E(i+1:end,:)], E_mean);
    end
    s2x = sum(temp.^2, 1);
    sPooled = sqrt((s2x + n1 .* varU) ./ dfe);
    ratio = (E_mean - meanU) ./ (sPooled .* sqrt(1/m1+1/n));
    P(i,:) = 2 * fast_tcdf(-abs(ratio),dfe);
end
E = Energy_cluster2;
for i = m+1:m+n
    % Compute p-value
    E_mean = (sumU-Energy_cluster2(i-m,:))./n1;
    if i==m+1
        temp = bsxfun(@minus, E(2:end,:), E_mean);
    elseif i==m+n
        temp = bsxfun(@minus, E(1:end-1,:), E_mean);
    else
        temp = bsxfun(@minus, [E(1:i-m-1,:);E(i-m+1:end,:)], E_mean);
    end
    s2y = sum(temp.^2, 1);
    sPooled = sqrt((m1 .* varH + s2y) ./ dfe);
    ratio = (meanH - E_mean) ./ (sPooled .* sqrt(1/m+1/n1));
    P(i,:) = 2 * fast_tcdf(-abs(ratio),dfe);
end



%%
function p = fast_tcdf(x,v)

v = v*ones(1,size(x,2));

p = NaN(size(x));
xsq = x.^2;
% For small v, form v/(v+x^2) to maintain precision
t = (v < xsq);
if any(t(:))
    p(t) = betainc(v(t) ./ (v(t) + xsq(t)), v(t)/2, 0.5, 'lower') / 2;
end
% For large v, form x^2/(v+x^2) to maintain precision
t = (v >= xsq);
if any(t(:))
    p(t) = betainc(xsq(t) ./ (v(t) + xsq(t)), 0.5, v(t)/2, 'upper') / 2;
end
% Make the result exact for the median.
p(x == 0) = 0.5;
%%