% This program computes the statistical results of the QCSSA program by
% doing a double leave-one-out validation, constraint to the given parameters.
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, H. L. Chan, R. Yong, S. Ranjitkar, A. Brook, G. Townsend, K. Chen, and L. M. Lui, 
%     "Tooth morphometry using quasi-conformal theory."
%     Pattern Recognition, 99, 107064, 2020.
%
% Copyright (c) 2019, Hei Long Chan

function [ TPR, TNR, CA ] = QCSSA_Stat( parameters, threshold, fold, Energy_cluster1, Energy_cluster2, method )


if nargin<6
    method = 'l2';
end
if fold==1
    fold = 'n';
end

% Compute the energy
E_c1 = QCSSA_RestoreEnergy( parameters, Energy_cluster1 );
E_c2 = QCSSA_RestoreEnergy( parameters, Energy_cluster2 );

% Initialize parameters
total_c1 = size(E_c1,1);
n = total_c1 + size(E_c2,1);
TPR = zeros(n,1);
TNR = zeros(n,1);
CA  = zeros(n,1);

% Compute p-value and find feature points
selection = QCSSA_Selection( E_c1, E_c2, threshold );
E_c1 = E_c1(:,selection);
E_c2 = E_c2(:,selection);

% Main program
for i = 1:total_c1
    E1 = E_c1;
    E1(i,:) = [];
    [TPR(i),TNR(i),CA(i)] = QCSSA_Classify(E1, E_c2, fold, method);
end
for i = total_c1+1:n
    E2 = E_c2;
    E2(i-total_c1,:) = [];
    [TPR(i),TNR(i),CA(i)] = QCSSA_Classify(E_c1, E2, fold, method);
end

% Print the final result
display(sprintf('**********************************************************************************************'))
display(sprintf(' '))
fprintf(' Parameter setting: ')
for ii = 1:length(parameters)
    fprintf('p%i= %f,    ',ii,parameters(ii));
end
fprintf('\n')
display(sprintf('         Database:       #. of cluster 1 = %i,    #. of cluster 2 = %i.',total_c1,n-total_c1))
display(sprintf('----------------------------------------------------------------------------------------------'))
display(sprintf('           Result:  max(TPR) = %f,    max(TNR) = %f,    max(CA) = %f',max(TPR),max(TNR),max(CA)))
display(sprintf('                   95up(TPR) = %f,   95up(TNR) = %f,   95up(CA) = %f',mean(TPR)+1.96*std(TPR)/sqrt(n),mean(TNR)+1.96*std(TNR)/sqrt(n),mean(CA)+1.96*std(CA)/sqrt(n)))
display(sprintf('                   mean(TPR) = %f,   mean(TNR) = %f,   mean(CA) = %f',mean(TPR),mean(TNR),mean(CA)))
display(sprintf('                  95low(TPR) = %f,  95low(TNR) = %f,  95low(CA) = %f',mean(TPR)-1.96*std(TPR)/sqrt(n),mean(TNR)-1.96*std(TNR)/sqrt(n),mean(CA)-1.96*std(CA)/sqrt(n)))
display(sprintf('                    min(TPR) = %f,    min(TNR) = %f,    min(CA) = %f',min(TPR),min(TNR),min(CA)))
display(sprintf('                    mdn(TPR) = %f,    mdn(TNR) = %f,    mdn(CA) = %f',median(TPR),median(TNR),median(CA)))
display(sprintf('                    std(TPR) = %f,    std(TNR) = %f,    std(CA) = %f',std(TPR),std(TNR),std(CA)))
display(sprintf(' '))
display(sprintf('**********************************************************************************************'))
