% Demo for computing a landmark-matching Teichmuller map between two tooth
% surfaces using the proposed method in [1].
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, H. L. Chan, R. Yong, S. Ranjitkar, A. Brook, G. Townsend, K. Chen, and L. M. Lui, 
%     "Tooth morphometry using quasi-conformal theory."
%     Pattern Recognition, 99, 107064, 2020.
%
% Copyright (c) 2019, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

addpath('code');

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load Tooth 1 and compute its rectangular conformal parameterization %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

data1 = load('tooth1.mat');
v1 = data1.v; f1 = data1.f; lm1 = data1.lm;

% Plot the source tooth surface
show_mesh(v1,f1);hold on;
plot3(v1(lm1,1),v1(lm1,2),v1(lm1,3),'ro','MarkerFaceColor','r');
title('Tooth 1'); view([140 -10]);

% Find the four corners of the rectangular domain
TR1 = TriRep(f1,v1); B1 = freeBoundary(TR1); bdy1 = B1(:,1);
corner1 = zeros(4,1);
corner1(1) = lm1(2);
corner1(3) = lm1(4);
[~,temp] = min((v1(bdy1,1) - v1(lm1(3),1)).^2 + (v1(bdy1,2) - v1(lm1(3),2)).^2 + (v1(bdy1,3) - v1(lm1(3),3)).^2);
corner1(2) = bdy1(temp);
[~,temp] = min((v1(bdy1,1) - v1(lm1(1),1)).^2 + (v1(bdy1,2) - v1(lm1(1),2)).^2 + (v1(bdy1,3) - v1(lm1(1),3)).^2);
corner1(4) = bdy1(temp);

% Compute the rectangular conformal parameterization
rect1 = rect_conformal_parameterization(v1,f1,corner1);

show_mesh(rect1,f1);hold on;
plot(rect1(lm1,1),rect1(lm1,2),'ro','MarkerFaceColor','r');
title('Rectangular conformal parameterization of tooth 1');

% Compute the curvature of the tooth surface
FV1.vertices = v1;
FV1.faces = f1;
[H1,K1] = patchcurvature(FV1,true);

show_mesh(v1,f1,H1); 
title('Mean curvature of tooth 1'); view([140 -10]);
show_mesh(v1,f1,K1); 
title('Gaussian curvature of tooth 1'); view([140 -10]);


%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load Tooth 2 and compute its rectangular conformal parameterization %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

data2 = load('tooth2.mat');
v2 = data2.v; f2 = data2.f; lm2 = data2.lm;

% Plot the target tooth surface
show_mesh(v2,f2);hold on; 
plot3(v2(lm2,1),v2(lm2,2),v2(lm2,3),'ro','MarkerFaceColor','r');
title('Tooth 2'); view([45 50]);

% Find the four corners of the rectangular domain
TR2 = TriRep(f2,v2); B2 = freeBoundary(TR2); bdy2 = B2(:,1);
corner2 = zeros(4,1);
corner2(1) = lm2(2);
corner2(3) = lm2(4);
[~,temp] = min((v2(bdy2,1) - v2(lm2(3),1)).^2 + (v2(bdy2,2) - v2(lm2(3),2)).^2 + (v2(bdy2,3) - v2(lm2(3),3)).^2);
corner2(2) = bdy2(temp);
[~,temp] = min((v2(bdy2,1) - v2(lm2(1),1)).^2 + (v2(bdy2,2) - v2(lm2(1),2)).^2 + (v2(bdy2,3) - v2(lm2(1),3)).^2);
corner2(4) = bdy2(temp);

% Compute the rectangular conformal parameterization
rect2 = rect_conformal_parameterization(v2,f2,corner2);

% Plot the conformal parameterization result
show_mesh(rect2,f2);hold on;
plot(rect2(lm2,1),rect2(lm2,2),'ro','MarkerFaceColor','r');
title('Rectangular conformal parameterization of tooth 2');

% Compute curvature
FV2.vertices = v2; FV2.faces = f2;
[H2,K2] = patchcurvature(FV2,true);
    
show_mesh(v2,f2,H2); 
title('Mean curvature of tooth 2'); view([45 50]);
show_mesh(v2,f2,K2); 
title('Gaussian curvature of tooth 2'); view([45 50]);

%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the Teichmuller map between the two rectangular domains %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

interior_landmark = lm1([1,3]); % indices of the interior landmarks
interior_landmark_target = rect2(lm2([1,3]),1:2); % target positions of the landmarks
height = max(rect2(:,2)); % target height of the rectangle

% Compute the Teichmuller map between the rectangles using QC Iteration
[tmap,tmap_bc] = rectangular_Teichmuller_map(rect1,f1,interior_landmark,interior_landmark_target,corner1,height);

% Computing the Teichmuller distance
d = 1/2*log((1+mean(abs(tmap_bc)))/(1-mean(abs(tmap_bc)))); 
fprintf('Teichmuller distnace between the teeth = %f\n',d);

% Construct the interpolants
Fx = scatteredInterpolant(rect2(:,1),rect2(:,2),v2(:,1),'linear');
Fy = scatteredInterpolant(rect2(:,1),rect2(:,2),v2(:,2),'linear');
Fz = scatteredInterpolant(rect2(:,1),rect2(:,2),v2(:,3),'linear');
H2_interp = scatteredInterpolant(v2(:,1),v2(:,2),v2(:,3),H2,'linear');
K2_interp = scatteredInterpolant(v2(:,1),v2(:,2),v2(:,3),K2,'linear');

% Find the surface T-map
tmap_surface = zeros(length(tmap),3);
tmap_surface(:,1) = Fx(tmap(:,1),tmap(:,2));
tmap_surface(:,2) = Fy(tmap(:,1),tmap(:,2));
tmap_surface(:,3) = Fz(tmap(:,1),tmap(:,2));

% Find the curvature difference
H_diff = abs(H1 - H2_interp(tmap_surface(:,1),tmap_surface(:,2),tmap_surface(:,3)));
K_diff = abs(K1 - K2_interp(tmap_surface(:,1),tmap_surface(:,2),tmap_surface(:,3)));

% Plot the Teichmuller map results
show_mesh(tmap,f1);hold on;
plot(tmap(lm1,1),tmap(lm1,2),'ro','MarkerFaceColor','r');
title('Landmark-matching Teichmuller map on rectangle');

show_mesh(tmap_surface,f1);hold on; view([45 50]);
plot3(tmap_surface(lm1,1),tmap_surface(lm1,2),tmap_surface(lm1,3),'ro','MarkerFaceColor','r');
title('Surface landmark-matching Teichmuller map');

% Plot the histogram of Beltrami coefficients
figure;hist(abs(tmap_bc),0:0.01:1);
xlabel('Norm of Beltrami coefficients');
ylabel('Number of faces');
title('Conformal distortion');

% Plot the surface Teichmuller mapping result with the curvature differences
show_mesh(tmap_surface,f1,H_diff); 
title('Absolute mean curvature difference between the teeth'); view([45 50]);
show_mesh(tmap_surface,f1,K_diff);
title('Absolute Gaussian curvature difference between the teeth'); view([45 50]);
